

// DIO01 - OCTAL DIGITAL I/O DEVICE


// Includes
#include "device.hpp"


// Preprocessor Directives
#pragma once


// Class DIO01
class DIO01 : public Device
{
public:

   // Class Exception
   class Exception;

   // Channels
   static const uint8_t CHANNELS = 8;

   // Direction
   static const bool DIRECTION_INPUT = false;
   static const bool DIRECTION_OUTPUT = true;

   // States
   static const bool STATE_LOW = false;
   static const bool STATE_HIGH = true;

   // Constructor
   DIO01(const IP::Address& address);
   DIO01(const Device& device);

   // Set Direction
   void direction(bool direction) const;
   void direction(uint8_t channel, bool direction) const;
   void direction(const std::set<uint8_t>& channel, bool direction) const;
   void direction(const std::map<uint8_t, bool>& direction) const;

   // Read
   bool read(uint8_t channel) const;
   std::map<uint8_t, bool> read(void) const;
   std::map<uint8_t, bool> read(const std::set<uint8_t>& channel) const;

   // Write
   void write(bool state) const;
   void write(uint8_t channel, bool state) const;
   void write(const std::set<uint8_t>& channel, bool state) const;
   void write(const std::map<uint8_t, bool>& state) const;

private:

   // DCP Message Types
   static const uint16_t _MESSAGE_SET_DIRECTION = 0x0300;
   static const uint16_t _MESSAGE_READ = 0x0301;
   static const uint16_t _MESSAGE_WRITE = 0x0302;
   static const uint16_t _MESSAGE_ERROR_CONFIGURATION = 0x0303;
   static const uint16_t _MESSAGE_DATA = 0x0304;
};


// Class Exception
class DIO01::Exception : public Device::Exception
{
public:

   // Class Configuration
   class Configuration;
};


// Class Configuration
class DIO01::Exception::Configuration : public DIO01::Exception
{
};
