

// IP 3.0.8


// Includes
#include <map>
#include <set>
#include <string>
#include <stdint.h>


// Preprocessor Directives
#pragma once


// Class IP
class IP
{
public:

   // Class Address (in Compliance with RFC 791 and RFC 5735)
   class Address;

   // Class Exception
   class Exception;

   // Get Adapters
   static const std::set<IP> adapter(void);

   // Get Gateway Address
   const Address gateway(void) const;

   // Get Host Address
   const Address host(void) const;

   // Get Adapter Name
   const std::string name(void) const;

   // Get Subnet Mask
   const Address subnet_mask(void) const;

private:

   // Network Adapter Info
   class _Info;

   // Get Network Adapter Info List
   static const std::map<std::string, _Info> _info(void);

   // Constructor
   IP(const std::string& name);

   // Operator <
   bool operator <(const IP& ip) const;

   // Variables
   std::string _name;

   // Friends
   friend std::less<IP>;
};


// Class Address (in Compliance with RFC 791 and RFC 5735)
class IP::Address
{
public:

   // Special Addresses
   static const Address BROADCAST;
   static const Address LOOPBACK;

   // Constructor
   Address(void);
   Address(uint32_t address);
   Address(uint8_t b0, uint8_t b1, uint8_t b2, uint8_t b3);
   Address(const std::string& address);

   // Convert
   operator uint32_t(void) const;
   operator const std::string(void) const;

   // Compare
   bool operator ==(const Address& address) const;
   bool operator !=(const Address& address) const;

   // Get Digit
   uint8_t& operator [](uint8_t n);
   const uint8_t& operator [](uint8_t n) const;

   // Check if broadcast
   bool broadcast(void) const;

   // Check if loopback
   bool loopback(void) const;

   // Check if in Subnet
   bool subnet(void) const;

private:

   // Variables
   uint8_t _b[4];
};


// Class Exception
class IP::Exception
{
public:

   // Class Address
   class Address;

   // Class Internal
   class Internal;

   // Class Network
   class Network;

private:

   // Virtual Function for RTTI
   virtual void _func() {}
};


// Class Address
class IP::Exception::Address : public IP::Exception
{
};


// Class Internal
class IP::Exception::Internal : public IP::Exception
{
};


// Class Network
class IP::Exception::Network : public IP::Exception
{
};
