

// VO01 - OCTAL VOLTAGE OUTPUT DEVICE


// Includes
#include <vector>
#include "device.hpp"


// Preprocessor Directives
#pragma once


// Class VO01
class VO01 : public Device
{
public:

   // Class Calibration
   class Calibration;

   // Class Exception
   class Exception;

   // Channels
   static const uint8_t CHANNELS = 8;

   // Constructor
   VO01(const IP::Address& address);
   VO01(const Device& device);

   // Calibrate
   void calibrate(void) const;
   void calibrate(const Calibration& calibration) const;

   // Control Voltage [V]
   void control(float voltage) const;
   void control(uint8_t channel, float voltage) const;
   void control(const std::set<uint8_t>& channel, float voltage) const;
   void control(const std::map<uint8_t, float>& voltage) const;

   // Stop Calibration
   void stop(void) const;

private:

   // DCP Message Types
   static const uint16_t _MESSAGE_CONTROL = 0x0800;
   static const uint16_t _MESSAGE_STOP = 0x0801;
   static const uint16_t _MESSAGE_ERROR_ABORTED = 0x0802;
   static const uint16_t _MESSAGE_SET_CALIBRATION = 0x0803;
   static const uint16_t _MESSAGE_BUSY = 0x0804;
   static const uint16_t _MESSAGE_ERROR_RESERVED = 0x0805;

   // Calibration Expiration [Days]
   static const uint16_t _CALIBRATION_EXPIRATION = 365;
};


// Class Calibration
class VO01::Calibration
{
public:

   // Constructor
   Calibration(void);
   Calibration(const std::string& str);

   // Convert to String
   operator std::string(void) const;

   // Parse String
   Calibration& operator =(const std::string& str);

   // Voltage Coefficient List
   const std::vector<float>& voltage(uint8_t channel) const;
   void voltage(uint8_t channel, const std::vector<float>& voltage);

private:

   // Polynomial Degree
   static const uint8_t _DEGREE = 7;

   // Variables
   std::vector<std::vector<float>> _voltage;
};


// Class Exception
class VO01::Exception : public Device::Exception
{
public:

   // Class Aborted
   class Aborted;

   // Class Reserved
   class Reserved;
};


// Class Aborted
class VO01::Exception::Aborted : public VO01::Exception
{
};


// Class Reserved
class VO01::Exception::Reserved : public VO01::Exception
{
};
